#================================================
#
#    wallpapoz_system.py - Wallpapoz 
#    Copyright (C) 2007 Akbar <akbarhome@gmail.com>
#
#================================================
#
#    This program is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program; if not, write to the Free Software
#    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
#================================================

## wallpapoz_system.py -- finds current desktop and changes wallpaper
# achieve goal by calling external program

import subprocess
import os
import sys
import string
import signal

class WallpapozSystem:

  # Define static variable to save the initial value of self.beryl
  static_check_beryl_called_p = False
  static_beryl = False

  # Also for total_workspaces
  static_finding_total_workspaces_called_p = False
  static_total_workspaces = 0

  def __init__(self):
    self.wallpaper_style = 'scaled'
    self.finding_screen_resolution()

    # Save the first result of self.beryl, and respawn daemon
    # if self.beryl value changed.
    self.check_beryl()
    if not WallpapozSystem.static_check_beryl_called_p :
      WallpapozSystem.static_check_beryl_called_p = True
      WallpapozSystem.static_beryl = self.beryl

    if self.beryl:
      self.finding_row_and_column()
    self.finding_total_workspaces()

    # Check if total_workspaces changed only here
    if WallpapozSystem.static_total_workspaces != self.total_workspaces:
      print "daemon_wallpapoz: the number of total workspaces changed during initialization process."
      print "daemon_wallpapoz: respawning daemon_wallpapoz."
      self.respawn_system()

  def exec_cmd_under_X(self, cmd):
    try:
      p = subprocess.Popen(
        cmd,
        stdout = subprocess.PIPE,
        stderr = subprocess.PIPE,
        shell = True
        )
    except OSError:
      print "daemon_wallpapoz: fork failed for %s, exiting." %cmd
      sys.exit(1)

    pstdout = p.stdout.read()
    pstderr = p.stderr.read()
    ret = p.wait()

    kill_daemon = False

    if (ret != 0) and (ret & 0xFF):
        print "daemon_wallpapoz: %s returned status %i." %(cmd, ret)
        print "daemon_wallpapoz: error message: %s" %pstderr
        kill_daemon = True

    if (cmd.find('xprop') >= 0) and (pstdout.find('no such atom') >= 0):
        kill_daemon = True

    if kill_daemon:
        ## No X resource available, kill daemon_wallpapoz
        print "daemon_wallpapoz: X resource seems no longer available."
        print "daemon_wallpapoz: killing daemon_wallpapoz."
        sys.exit(1)

    return pstdout

  def set_style(self, style):
    self.wallpaper_style = style

  ## class method to find monitor resolution
  def finding_screen_resolution(self):
    raw_resolution = self.exec_cmd_under_X('xwininfo -root')
    start_width = raw_resolution.find('Width')
    end_width = raw_resolution.find('\n',start_width)
    start_height = raw_resolution.find('Height')
    end_height = raw_resolution.find('\n', start_height)
    self.screen_width = int(raw_resolution[start_width+7:end_width])
    self.screen_height = int(raw_resolution[start_height+8:end_height])

  ## class method to find amount of workspaces in user desktop
  def finding_total_workspaces(self):
    if self.beryl:
      self.total_workspaces = self.row_workspaces * self.column_workspaces
    else:
      self.total_workspaces = int(self.exec_cmd_under_X("xprop -root _NET_NUMBER_OF_DESKTOPS")[36:38])

    # Save this total_workspaces value
    if not WallpapozSystem.static_finding_total_workspaces_called_p:
      WallpapozSystem.static_finding_total_workspaces_called = True
      WallpapozSystem.static_total_workspaces = self.total_workspaces

  ## class method to find if user use beryl in his system or not
  def check_beryl(self):
    raw_geometry = self.exec_cmd_under_X('xprop -root _NET_DESKTOP_GEOMETRY')
    # output of xprop -root _NET_DESKTOP_GEOMETRY is '_NET_DESKTOP_GEOMETRY(CARDINAL) = 1024, 768\n'
    # and we just need the '1024, 768' part
    raw_geometry = raw_geometry[34:raw_geometry.find('\n')]
    geometry = string.split(raw_geometry, ', ')
    self.geometry_width = int(geometry[0])
    self.geometry_height = int(geometry[1])
    self.beryl = False
    if self.geometry_width!=self.screen_width:
      self.beryl = True
    elif self.geometry_height!=self.screen_height:
      self.beryl = True

    # respawn daemon if self.beryl value changed
    if WallpapozSystem.static_check_beryl_called_p:
      if WallpapozSystem.static_beryl != self.beryl:
        print "Compiz status changed, respawning daemon_wallpapoz."
        self.respawn_system()

  ## class method to find how many rows and columns of workspaces
  def finding_row_and_column(self):
    self.row_workspaces = self.geometry_height / self.screen_height
    self.column_workspaces = self.geometry_width / self.screen_width

  ## class method to find how many workspaces there are in user desktop
  def get_total_workspaces(self):
    return self.total_workspaces

  ## class method to know what workspace we are in now
  def current_desktop(self):
    if self.beryl:
      raw_viewport = self.exec_cmd_under_X('xprop -root _NET_DESKTOP_VIEWPORT')
      # output of xprop -root _NET_DESKTOP_VIEWPORT is '_NET_DESKTOP_VIEWPORT(CARDINAL) = 1024, 768\n'
      # and we just need the '1024, 768' part
      raw_viewport = raw_viewport[34:raw_viewport.find('\n')]
      viewport = string.split(raw_viewport, ', ')
      x_pos = int(viewport[0]) / self.screen_width
      y_pos = int(viewport[1]) / self.screen_height
      workspace = x_pos + self.column_workspaces * y_pos
    else:
      raw_workspace = self.exec_cmd_under_X('xprop -root _NET_CURRENT_DESKTOP')
      workspace = int(raw_workspace[33] + raw_workspace[34])
    return workspace

  ## class method to change desktop wallpaper
  def change_wallpaper(self, wallpaper):
    os.system('gconftool-2 -t string -s /desktop/gnome/background/picture_filename ' + 
	'"' + wallpaper + '"' + ' -s /desktop/gnome/background/picture_options ' + 
	self.wallpaper_style)
    # And for gsettings
    status=os.system("which gsettings &>/dev/null")
    if status == 0:
        basecommand='gsettings set org.gnome.desktop.background '
        os.system(basecommand + 'picture-uri ' + "'\"" + 'file://' + wallpaper + "\"'")
        os.system(basecommand + 'picture-options ' + self.wallpaper_style)
        os.system(basecommand + 'draw-background true')

  ## class method to detect that we have changed workspace or not
  def has_changed(self, previous_desktop, cur_desk):
    if previous_desktop != cur_desk:
      # if we move to workspace that we don't use, just ignore
      if cur_desk >= self.total_workspaces:
	return False
      return True
    else:
      return False

  def respawn_system(self):
    new_argv = []
    new_argv.extend(sys.argv)
    os.execvp('daemon_wallpapoz', new_argv)
    print "os.execvp failed, exiting..."
    sys.exit(1)

  def prevent_multiple_start(self):
    cpid = os.getpid()
    cid = os.getuid()

    wallpapoz_cmd = "pgrep -U %i -f daemon_wallpapoz" %cid
    python_cmd = "pgrep -U %i -f python" %cid

    wallpapoz_id_list = os.popen(wallpapoz_cmd).read().splitlines()
    python_id_list = os.popen(python_cmd).read().splitlines()

    for id_list in wallpapoz_id_list:
      if ( id_list in python_id_list ) and ( int(id_list) != cpid) :
        os.kill(int(id_list), signal.SIGTERM)

